#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.Hive.Contracts.BusinessObjects;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.PluginInfrastructure;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.ServiceModel;
using HeuristicLab.Hive.Server.Core.InternalInterfaces;
using System.Transactions;
using HeuristicLab.Hive.Server.LINQDataAccess;
using HeuristicLab.Hive.Server.DataAccess;

namespace HeuristicLab.Hive.Server.Core {
  [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall, ConcurrencyMode = ConcurrencyMode.Multiple)]
  public class ClientFacade : IClientFacade {

    public ClientFacade() {
    }

    private IClientCommunicator clientCommunicator = ServiceLocator.GetClientCommunicator();

    private IContextFactory contextFactory = ServiceLocator.GetContextFactory();

    #region IClientCommunicator Members

    public Response Login(ClientDto clientInfo) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.Login(clientInfo);
      }
    }

    public ResponseHB ProcessHeartBeat(HeartBeatData hbData) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.ProcessHeartBeat(hbData);
      }
    }

    public ResponseJob SendJob(Guid clientId) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.SendJob(clientId);
      }
    }

    public ResponseResultReceived StoreFinishedJobResult(Guid clientId, Guid jobId, byte[] result, double percentage, Exception exception) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.StoreFinishedJobResult(clientId, jobId, result, percentage, exception);
      }
    }

    public Response Logout(Guid clientId) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.Logout(clientId);
      }
    }

    public Response IsJobStillNeeded(Guid jobId) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.IsJobStillNeeded(jobId); 
      }
    }

    public ResponsePlugin SendPlugins(List<HivePluginInfoDto> pluginList) {
      return clientCommunicator.SendPlugins(pluginList);      
    }

    public ResponseResultReceived ProcessSnapshot(Guid clientId, Guid jobId, byte[] result, double percentage, Exception exception) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.ProcessSnapshot(clientId, jobId, result, percentage, exception); 
      }
    }


    public ResponseCalendar GetCalendar(Guid clientId) {
      using (contextFactory.GetContext()) {
        return clientCommunicator.GetCalendar(clientId); 
      }
    }

    public Response SetCalendarStatus(Guid clientId, CalendarState state) { 
      using (contextFactory.GetContext()) {
        return clientCommunicator.SetCalendarStatus(clientId, state);
      }
    }
    #endregion

    #region IClientFacade Members

    /// <summary>
    /// Do not use automatic transactions here
    /// </summary>
    public Stream SendStreamedJob(Guid clientId) {
      using (contextFactory.GetContext(false)) {
        MultiStream stream = new MultiStream();

        ResponseJob job = null;

        job = ServiceLocator.GetClientCommunicator().SendJob(clientId);

        //first send response
        stream.AddStream(new StreamedObject<ResponseJob>(job));

        IJobManager jobManager = ServiceLocator.GetJobManager();
        IInternalJobManager internalJobManager = (IInternalJobManager)jobManager;

        //second stream the job binary data
        MemoryStream memoryStream = new MemoryStream();
        if (job.Job != null)
          stream.AddStream(new MemoryStream(internalJobManager.GetSerializedJobDataById(job.Job.Id)));

        OperationContext clientContext = OperationContext.Current;
        clientContext.OperationCompleted += new EventHandler(delegate(object sender, EventArgs args) {
          if (stream != null) {
            stream.Dispose();
          }
        });

        return stream;
      }
    }

    public Stream SendStreamedPlugins(List<HivePluginInfoDto> pluginList) {
      return new StreamedObject<ResponsePlugin>(this.SendPlugins(pluginList));
    }

    public ResponseResultReceived StoreFinishedJobResultStreamed(Stream stream) {
      using (contextFactory.GetContext()) {
        return ((IInternalClientCommunicator)clientCommunicator).ProcessJobResult(stream, true);
      }
    }

    public ResponseResultReceived ProcessSnapshotStreamed(Stream stream) {
      using (contextFactory.GetContext()) {
        return ((IInternalClientCommunicator)clientCommunicator).ProcessJobResult(stream, false);  
      }
    }
    #endregion
  }
}
